import 'package:comunic/helpers/groups_helper.dart';
import 'package:comunic/helpers/users_helper.dart';
import 'package:comunic/lists/groups_list.dart';
import 'package:comunic/lists/posts_list.dart';
import 'package:comunic/lists/users_list.dart';
import 'package:comunic/models/post.dart';
import 'package:comunic/ui/screens/conversation_screen.dart';
import 'package:comunic/ui/tiles/post_tile.dart';
import 'package:comunic/utils/intl_utils.dart';
import 'package:comunic/utils/ui_utils.dart';
import 'package:flutter/material.dart';

/// Posts list widget
///
/// Displays a list of posts
///
/// @author Pierre HUBERT

class PostsListWidget extends StatefulWidget {
  final Future<PostsList> Function() getPostsList;
  final bool showPostsTarget;

  const PostsListWidget({
    Key key,
    @required this.getPostsList,
    @required this.showPostsTarget,
  })  : assert(getPostsList != null),
        assert(showPostsTarget != null),
        super(key: key);

  @override
  State<StatefulWidget> createState() => _PostsListWidgetState();
}

class _PostsListWidgetState extends State<PostsListWidget> {
  // Helpers
  final UsersHelper _usersHelper = UsersHelper();
  final GroupsHelper _groupsHelper = GroupsHelper();

  // Class members
  PostsList _list;
  UsersList _users;
  GroupsList _groups;

  set error(ErrorLevel err) => setState(() => _error = err);

  ErrorLevel _error = ErrorLevel.NONE;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    _loadPostsList();
  }

  void _loadError() =>
      error = _list == null ? ErrorLevel.MAJOR : ErrorLevel.MINOR;

  /// Load the list of posts
  Future<void> _loadPostsList() async {
    final list = await widget.getPostsList();

    if (list == null) return _loadError();

    final users = await _usersHelper.getList(list.usersID);

    if (users == null) return _loadError();

    final groups = await _groupsHelper.getList(list.groupsID);

    if(groups == null) return _loadError();

    setState(() {
      _list = list;
      _users = users;
      _groups = groups;
    });
  }

  Widget _buildErrorCard() {
    return buildErrorCard(tr("Could not get the list of posts !"));
  }

  Widget _buildListView() {
    return ListView.builder(
      itemCount: _list.length,
      itemBuilder: (c, i) => PostTile(
            post: _list[i],
            usersInfo: _users,
            groupsInfo: _groups,
            onDeletedPost: _removePost,
            showPostTarget: widget.showPostsTarget,
          ),
    );
  }

  @override
  Widget build(BuildContext context) {
    if (_error == ErrorLevel.MAJOR) return _buildErrorCard();
    if (_list == null) return buildCenteredProgressBar();
    return _buildListView();
  }

  void _removePost(Post post) => setState(() => _list.remove(post));
}
